#!/usr/bin/env python
# coding: latin-1
# Autor:   Ingmar Stapel
# Datum:   20160731
# Version:   2.0
# Homepage:   http://custom-build-robots.com
# GPS Programm fuer das autonome Fahren des Roboter Autos

import time, smbus, os

# Python math Bibliothek importieren
from math import *

# GPS Bibliothek importieren
from gps import *

# Threads Bibliothek importieren
from threading import Thread

# Das Programm L298NHBridge.py wird als Modul geladen.
import L298NHBridge as HBridge

# Sense HAT Bibliothek importieren
from sense_hat import SenseHat

# Initialisieren des SenseHat
sense = SenseHat()

# Das Programm gps_calculate_module.py wird als Modul geladen.
import gpscalculatemodule as GPSCalc

# Es wird die Python Klasse ledmatrix importiert.
import ledmatrix as matrix

# Definition der globalen Stopp Variablen
global stopp_gps
global stopp_print
global stopp_bearing
global time_stopp

# Definition der globalen Koordinaten Variablen
global act_latitude
global act_longitude
global start_latitude
global start_longitude
global last_pos_latitude
global last_pos_longitude

# Ziel-Koordinate
global target_lat
global target_lon

# Variablen fuer das Gyroskop
global z

# Globale Ergebnis Variablen
global h_bearing
global distance

# Globale Nachrichten Variable
global msg
global mag

# Initiale Belegung von Variablen
act_latitude = 0
act_longitude = 0
start_latitude = 0
start_longitude = 0
last_pos_latitude = 0
last_pos_longitude = 0

z = 0

h_bearing = 0
distance = 0
msg = "Start"
mag = "J"

def readgps():
   # Anlegen der GPS-Session, damit der GPS Empfaenger  
   # ausgelesen werden kann.
   session = gps(mode=WATCH_ENABLE)
   # count ist eine Hilfsvariable um jede 1 Sekunde die
   # aktuellen GPS-Koordinaten zwischenzuspeichern in den beiden
   # Varialben last_pos_latitude und last_pos_longitude
   count = 0
   global stopp_gps
   global time_stopp
   
   # Variablen fuer Zwischenspeicherung der GPS-Koordinaten
   # Aktuelle Position
   global act_latitude
   global act_longitude
   # Start Position
   global start_latitude
   global start_longitude
   # Wird alle 10 Sekunden aktualisiert
   global last_pos_latitude
   global last_pos_longitude

   # Variable für den Kurswinkel
   global h_bearing
   # Variable fuer den Abstand
   global distance

   # initialisieren von Variablen
   start_latitude = 0
   start_longitude = 0
   
   # Wenn stopp_gps den Wert 1 hat wird die While Schleife
   # beendet und somit der Thread das Programm.
   stopp_gps = 0
      
   while stopp_gps == 0:
      if time_stopp < time.time():
         stopp_gps = 1
      # Holt die aktuellen GPS-Informationen in die 
      # Variable session
      session.next()
      
      # Erst wenn ein GPS Fix vorhanden ist geht das Programm 
      # weiter. Solange wird nach einer gueltigen GPS 
      # Koordinate gesucht.
      if session.fix.latitude != 0.0 and session.fix.longitude:
         # Belegung der Start Variable mit GPS Koordinaten
         if start_latitude == 0:
            start_latitude = session.fix.latitude
            start_longitude = session.fix.longitude
            # initiale Belegung der Variable für die letzte 
            # bekannte Position 1 Sec in der Vergangenheit
            last_pos_latitude = session.fix.latitude
            last_pos_longitude = session.fix.longitude
         # Aktuelle Position
         act_latitude = session.fix.latitude
         act_longitude = session.fix.longitude

      # Jede Sekunde ist count%10 wahr
      if count%10 == 0:
         if session.fix.latitude != 0.0 and \
         session.fix.longitude != 0.0:
            # Alle 10 Durchlaeufe bzw. ca. jede Sekunde  
            # wird die Variable mit der letzten GPS 
            # Postition aktualisiert.
            last_pos_latitude = session.fix.latitude
            last_pos_longitude = session.fix.longitude      
      count = 0
         
      time.sleep(0.1)
      count += 1
      
# Auslesen des Gyroskopes des Sense HAT 
def gyroscope():
# Variablen als global definieren
   global z
   global stopp_gyroscope
   stopp_gyroscope = 0

   # Aktivieren des Gyroskop
   sense.set_imu_config(False, True, False)

   # So lange stopp = 0 ist arbeitet dieser Thread in der
   # Endlosschleife
   while stopp_gyroscope == 0:
      if time_stopp < time.time():
         stopp_gyroscope = 1
         sense.set_imu_config(False, False, False)
   
      # Auslesen der horizontalen Lage des Roboter Autos.
      z = sense.get_orientation()['yaw']
      time.sleep(0.01)   

# Drehen um einen bestimmten Gradwert 
def turn_robot(value):
   global z
   global stopp_turn
   global turn
   global turn_delta
   global speed
   global stopp_gyroscope
   stopp_turn = 0
   global msg
   
   # initiale Drehgeschwindigkeit
   speed = 1
   turn = value
   stopp_gyroscope = 0
   
   # Kurz warten, damit der Thread auch gestartet hat und die
   # Gyroskop Werte vorliegen.
   time.sleep(0.5)
      
   for i in range(5):
      msg = "Vor der For Schleife"
      # Hier wird nur das Magenetometer angesprochen.
      sense.set_imu_config(True, False, False) 
      msg = "Nach sense"      
      # Get the direction of North
      north = sense.get_compass()
      degree = float(north)
      msg = "Kompass Kalibrierung in ", 5 - i, "s akt. Winkel: "\
      ,round(degree,0)
      time.sleep(1)
   # Starten des Threads, welcher das Gyroskop ausliest
   t_gyroscope = Thread(target=gyroscope)
   t_gyroscope.start()
   
   # So lange stopp = 0 arbeitet dieser Thread in der
   # Endlosschleife und richtet das Roboter-Auto aus.
   msg = "Starte Drehung des Roboter-Autos: "
   time.sleep(4)
   while stopp_turn == 0:
      if time_stopp < time.time():
         stopp_turn = 1      

      msg = "Aktueller Gyroskop Wert: ",round(z,0)

      if turn <= 180:
         if turn - z > 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed)
         elif turn - z < 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)      
      elif turn > 180:
         if z - turn > 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)         
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)
         elif z - turn < 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)         
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed)         

      # Berechnung der aktuellen Abweichung
      turn_delta = turn - z   

      # Wenn die Abweichung zum Ziel lediglich 
      # +-8 Grad betraegt verlangsamen die Motoren 
      if z - 15 < turn < z + 15:
         speed = 0.8         

      # Wenn die Abweichung zum Ziel lediglich 
      # +-2 Grad betraegt stoppen die Motoren 
      if z - 0.5 < turn < z + 0.5:
         matrix.display_pixels("finish")
         stopp_gyroscope = 1
         stopp_turn = 1
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)
         sense.set_imu_config(False, False, False)

      time.sleep(0.04)
   msg = "Drehung abgeschlossen: ",round(turn_delta,0)
   time.sleep(3)   

# Diese Funktion uebernimmt die Steuerung des Roboter Autos,
# damit es GPS Koordianten abfahren kann.
def drive_robot():
   global stopp_drive
   global act_latitude
   global act_longitude
   global start_latitude
   global start_longitude   
   global stopp_turn
   global target_lat
   global target_lon
   global msg
   global mag
   global distance
   global h_bearing
   # initiale Fahrgeschwindigkeit
   speed = 1
   stopp_drive = 0
   stopp_ausrichten = 0
   distance = 0
   
   angle_b = 0
   angle_c = 0
   angle_d = 0

   # Hier muessen Sie ihre eigenen Weg-Punkte im Programm 
   # hinterlegen. Passen Sie die GPS-Koordinaten nicht an,
   # so faehrt das Roboter-Auto zum Norisring!

   # Hier können Sie die Ziel-Koordinate fuer Tests ueberschreiben.
   # Das hat den Vorteil, dass Sie wenn Sie testen nicht immer
   # die Koordinaten beim Programmaufruf von Hand eingeben muessen.
   # target_lat = 48.24983124384924
   # target_lon = 11.65604102108043         
   
   stopp_ausrichten = 0   

   # Es wird der Sensor Magenetometer aktiviert.
   sense.set_imu_config(True, False, False)        

   # Warten, damit sich das Magenetometer ausrichten kann.
   time.sleep(5)
   # Es wird die Gradzahl Richtung Norden ausgelesen.
   north = sense.get_compass()

   # Die Gradzahl soll als Float Wert uebergeben werden.
   angle_b = float(north)
   
   # Sense-HAT Sensoren wieder freigeben.
   sense.set_imu_config(False, False, False)
   msg = "Winkel b von Magnetometer:", round(angle_b,0)

   time.sleep(3)
   
   msg = "Berechne als naechstes den Winkel zum Ziel"
   time.sleep(3)   
   # Es wird berechent wie groß der Winkel zum Ziel ist. Dazu 
   # werden die Aktuellen Koordinaten und die Zielkoordinaten
   # fuer die Berechnung genommen. 

   distance, angle_c = GPSCalc.calc(act_latitude, \
   act_longitude, target_lat, target_lon)
   h_bearing = round(angle_c,0)
   msg = "Winkel c vor Berechnung:", round(angle_c,0)
   time.sleep(5)   
   
   # Wertebereich des Kurswinkels beachten und wenn negativ
   # den Winkel umrechnen damit das Programm mit desem
   # weiter arbeiten kann.
   if angle_c < 0:
      angle_c += 360
   
   msg = "Winkel c nach Berechnung:", round(angle_c,0)
   time.sleep(5)
   angle_d = angle_c
      
   # Uebergabe des berechenten Winkels an die Printfunktion.   
   msg = "Dreh-Winkel des Roboter Autos: ", round(angle_d,0)   
   time.sleep(5)
   
   # Das Roboter Auto dreht auf die Zielkoordiante.   
   turn_robot(angle_d)

   # Warten bis das Roboter Auto fertig gedreht hat. Dazu wird
   # auf die globale Variable stopp_turn zurueck gegriffen.
   while stopp_turn == 0:
      time.sleep(0.01)      

   # Uebergabe der Information an die Druckfunktion das sich das
   # Roboter Auto fertig geddreht hat..      
   msg = "Fahre los zum Ziel..."
   time.sleep(2.5)   
   
   # Fahre ab jetzt einfach gerade aus ohne weiterer Überprüfung
   # der richtigen Ausrichtung des Roboter -Autos zum Ziel bis der
   # Abstand zum Ziel < 4m ist.
   while stopp_ausrichten == 0:
      if time_stopp < time.time():
         stopp_ausrichten = 1
      if start_latitude != 0 and start_longitude != 0:
         distance, angle_c = GPSCalc.calc(act_latitude, \
		 act_longitude, target_lat, target_lon)
         h_bearing = angle_c
         msg = "Fahre zum Ziel - Abstand: ", round(distance,1)
      if distance < 4:
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)
         msg = "Abstand < 4m Ende geradeaus fahren Ziel \
         erreicht: ", distance
         stopp_ausrichten = 1
      else:
         matrix.display_pixels("arrow")
         matrix.display_rotate(90)   
         HBridge.setMotorLeft(speed)
         HBridge.setMotorRight(speed)
      time.sleep(0.01)

# Diese Funktion aktualisiert die Textausgabe im Terminal Fenster      
def printdata():
   global time_stopp
   global stopp_print
   global act_latitude
   global act_longitude
   global start_latitude
   global start_longitude
   global h_bearing   
   global distance
   global msg
   
   stopp_print = 0
   laufzeit = 0
   while stopp_print == 0:   
      if time_stopp < time.time():
         stopp_print = 1

      os.system('clear')
      laufzeit = time_stopp - time.time()
      print "---------------------------------------"
      print ":::::   Status autonomes Fahren   :::::"
      print "---------------------------------------"   
      print "Start Laengengrad:          ",start_latitude
      print "Start Breitengrad:          ",start_longitude
      print "Aktueller Laengengrad:      ",act_latitude
      print "Aktueller Breitengrad:      ",act_longitude
      print "Horizontaler Winkel:        ",round(h_bearing,1)
      print "Abstand in m:               ",round(distance,1)
      print "Gyroskop Winkel z (drehen): ",round(z,1)
      print "Programm Information:       ",msg
      print "Programmlaufzeit in sec:    ",round(laufzeit,1)
      time.sleep(0.5)   

# Der Anwender wird aufgefordert den Laengengrad und 
# Breitengard einzugeben. Weiter wird der Anwender gefragt
# ob das Raspberry Pi Sense-HAT Magnetometer verwendet
# werden soll um die Ausrichtung nach Norden zu ermitteln.
try:
   laufzeit=input("Programmlaufzeit in Minuten eingeben: ")
   target_lat=input("Geben Sie den Laengengrad ein:")
   target_lon=input("Geben Sie den Breitengrad ein:")
except ValueError: 
   sys.exit()

time_stopp = laufzeit*60 + time.time()
# Starten der Threads die fuer das Programm aktiv sein muessen.
t_readgps = Thread(target=readgps)
t_readgps.start()

t_print = Thread(target=printdata)
t_print.start()   

drive_robot()

# Ende des Programmes