#!/usr/bin/env python
# coding: latin-1
# Autor:   Ingmar Stapel
# Datum:   20160731
# Version:   2.0
# Homepage:   http://custom-build-robots.com
# Programm fuer die Entfernungsmessung mit zwei
# SRF08 Ultraschall Sensoren.

import smbus, time, os

# Setzen des SMBus Geraetes.
dev = smbus.SMBus(1)

# Definition der I2C Adressen der beiden Ultraschall Sensoren 
address1 = 0x70 # vornen
address2 = 0x72 # hinten

# Die Variablen light1 und light2 geben die Helligkeit aus.
light1 = 0
light2 = 0

# Festlegen des sogenannten Ranging Mode. Dieser Modus bestimmt  
# die Einheit in der die Entfernung vom Sensor zurueck gegeben
# wird. Die folgenden Modus stehen zur Auswahl:
# 0x50    Mode - Ergebnis in Inches
# 0x51    Mode - Ergebnis in Zentimeter
# 0x52    Mode - Ergebnis in Millisekunden
# Das Programm soll die Entfernung in cm ausgeben.
mode = 0x51

# LOOP legt die Anzahl der Einzelmessungen fest. 
LOOP = 100

# intervall legt die Wartezeit zwischen den Messungen fest.
intervall = 0.1

# Definition des linken und rechten Schwellwertes
min_dist = 120
max_dist = 200

# Initialisierung der Entfernungmessung mit den Eingabevarialben:
# sensor_id = Adresse des Sensors 
# mode      = setzen des Range Mode
def start_ranging(sensor_id, mode):
      # Es wird in Register 1 der 
      # gain Wert geschrieben.
      dev.write_byte_data(sensor_id, 1, 0x06)
      # Es wird in den Range Register 2 der 
      # range Wert geschrieben.
      dev.write_byte_data(sensor_id, 2, 430)
      # Der Aufruf des Registers 0 teilt dem Sensor mit
      # die Messung zu beginnen mit dem festgelegten Modus.
      dev.write_byte_data(sensor_id, 0, mode)
      return -1

# Diese Funktion liest die Software Version des Sensors aus.
def software_version(sensor_id):
        version = dev.read_byte_data(sensor_id, 0)
        return version      

# Diese Funktion liest die Helligkeit am Sensor aus.      
def light_sensor(sensor_id):
        gain_value = dev.read_byte_data(sensor_id, 1)
        return gain_value
      
# Diese Funktion liest die gemessene Entfernung des Sensors aus.
def distance(sensor_id):
      highByte2 = dev.read_byte_data(sensor_id, 2)
      lowByte3 = dev.read_byte_data(sensor_id, 3)

      # Berechnen der Entfernung aus high und low Byte
      calc_range = (highByte2 << 8) + lowByte3
      return calc_range

# Diese Schleife fuehrt die Messung so oft durch wie dies durch
# die Variable LOOP vorgegeben ist.
for x in range(0, LOOP):
      # startet die Messung für den Sensor mit der address1
      start_ranging(address1, mode)
      time.sleep(intervall)
      # startet die Messung für den Sensor mit der address2
      start_ranging(address2, mode)
      time.sleep(intervall)
      # Ruft die Funktione zur Enfernungmessung fuer dist1
      dist1 = distance(address1)
      # Ruft die Funktione zur Enfernungmessung fuer dist2
      dist2 = distance(address2)
      
      # Ausgabe der gemessenen Entfernungen ohne Messfehler.
      if dist1 > min_dist and dist1 < max_dist:
         print "Abstand vorne:", dist1,\
         " cm", " - Helligkeit vorne: ",\
         light_sensor(address1)
      if dist2 > min_dist and dist2 < max_dist:
         print "Abstand hinten:", dist2,\
         " cm", " - Helligkeit hinten: ",\
         light_sensor(address2)

print "Software Version Sensor 1: ", software_version(address1)
print "Software Version Sensor 2: ", software_version(address2)
# End des Programmes
